@@ -1,7 +1,7 @@
 ;; -*- mode: scheme; coding: utf-8 -*-
 
 ;;;;
-;;;; Copyright (C) 2016 - 2017
+;;;; Copyright (C) 2016 - 2018
 ;;;; Free Software Foundation, Inc.
 
 ;;;; This file is part of GNU Guile-CV.
@@ -29,6 +29,7 @@
 (define-module (cv support f32vector)
   #:use-module (ice-9 match)
   #:use-module (ice-9 receive)
+  #:use-module (ice-9 format)
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-4)
   #:use-module (system foreign)
@@ -50,16 +51,23 @@
             f32vector-add-vectors
             f32vector-subtract-value
             f32vector-subtract-vectors
-            f32vector-multiply-value
+            f32vector-times-value
+            f32vector-times-vectors
+            f32vector-mtimes
             f32vector-divide-value
+            f32vector-divide-vectors
+            f32vector-invert
             f32vector-and-vectors
             f32vector-or-vectors
             f32vector-xor-vectors
             f32vector-=-vectors?
             f32vector-binary-vectors?
             f32vector-is-a-seed?
+            f32vector-scale
+            f32vector->s32vector
 
             ;; Pure scheme code
+            f32vector-display
             f32vector-reduce
             f32vector-mean
             f32vector-std-dev
@@ -77,10 +85,7 @@
 	    f32vector=?
 	    f32vector-pred-at-offset?
 	    f32vector-index
-	    f32vector-count-distinct
-            f32vector-invert
-            f32vector-matrix-multiply
-            f32vector->s32vector))
+	    f32vector-count-distinct))
 
 
 ;;;
@@ -220,14 +225,39 @@
                                     n-chan)
     to)))
 
-(define* (f32vector-multiply-value channel val to #:key (n-cell #f))
+(define* (f32vector-times-value channel val to #:key (n-cell #f))
   (let ((n-cell (or n-cell (f32vector-length channel))))
-    (f32vector-multiply-value-c (bytevector->pointer channel)
-                                n-cell
-                                val
-                                (bytevector->pointer to))
+    (f32vector-times-value-c (bytevector->pointer channel)
+                             n-cell
+                             val
+                             (bytevector->pointer to))
     to))
 
+(define (f32vector-times-vectors to n-cell channels)
+  (receive (maker setter!)
+      (get-v-ptr-maker-setter)
+    (let* ((n-chan (length channels))
+           (v-ptr (maker n-chan 0)))
+      (for-each (lambda (chan i)
+                  (setter! v-ptr i
+                           (pointer-address (bytevector->pointer chan))))
+          channels
+        (iota n-chan))
+      (f32vector-times-vectors-c (bytevector->pointer to)
+                                 n-cell
+                                 (bytevector->pointer v-ptr)
+                                 n-chan)
+    to)))
+
+(define (f32vector-mtimes v1 width-1 height-1 v2 width-2 to)
+  (f32vector-mtimes-c (bytevector->pointer v1)
+                      width-1
+                      height-1
+                      (bytevector->pointer v2)
+                      width-2
+                      (bytevector->pointer to))
+  to)
+
 (define* (f32vector-divide-value channel val to #:key (n-cell #f))
   (if (= val 0.0)
       (error "Attempt to divide by 0")
@@ -238,6 +268,29 @@
                                   (bytevector->pointer to))
         to)))
 
+(define (f32vector-divide-vectors to n-cell channels)
+  (receive (maker setter!)
+      (get-v-ptr-maker-setter)
+    (let* ((n-chan (length channels))
+           (v-ptr (maker n-chan 0)))
+      (for-each (lambda (chan i)
+                  (setter! v-ptr i
+                           (pointer-address (bytevector->pointer chan))))
+          channels
+        (iota n-chan))
+      (f32vector-divide-vectors-c (bytevector->pointer to)
+                                  n-cell
+                                  (bytevector->pointer v-ptr)
+                                  n-chan)
+    to)))
+
+(define* (f32vector-invert channel to #:key (n-cell #f))
+  (let ((n-cell (f32vector-length channel)))
+    (f32vector-invert-c  (bytevector->pointer channel)
+                         n-cell
+                         (bytevector->pointer to))
+    to))
+
 (define (f32vector-and-vectors to n-cell channels)
   (receive (maker setter!)
       (get-v-ptr-maker-setter)
@@ -323,11 +376,43 @@
                             (bytevector->pointer s-chan))
      0))
 
+(define* (f32vector-scale channel n-max to #:key (n-cell #f) (p-max 255))
+  (let ((n-cell (or n-cell (f32vector-length channel))))
+    (f32vector-scale-c (bytevector->pointer channel)
+                       n-cell
+                       p-max
+                       n-max
+                       (bytevector->pointer to))
+    to))
+
+(define* (f32vector->s32vector v #:key (n-cell #f))
+  (let* ((n-cell (or n-cell (f32vector-length v)))
+	 (to (make-s32vector n-cell)))
+    (f32vector-to-s32vector-c (bytevector->pointer v)
+                              n-cell
+                              (bytevector->pointer to))
+    to))
+
 
 ;;;
 ;;; Pure scheme code
 ;;;
 
+(define* (f32vector-display v #:key (proc #f) (n-cell #f)
+                            (port (current-output-port)))
+  (let ((proc (if proc
+                  proc
+                  (lambda (val)
+                    (if (float>=? val 1000.0 0)
+                        (format #f "~9e" val)
+                        (format #f "~9,5,,,f" val)))))
+        (n-cell (or n-cell
+                    (f32vector-length v))))
+    (do ((i 0
+            (+ i 1)))
+        ((= i n-cell))
+      (format port "  ~A~%" (proc (f32vector-ref v i))))))
+
 (define* (f32vector-reduce v proc default #:key (n-cell #f))
   (let ((n-cell (or n-cell
                     (f32vector-length v))))
@@ -550,10 +635,10 @@
 ;;; Matrix ops
 ;;;
 
-(define (f32vector-matrix-multiply v1 width-1 height-1 v2 width-2)
+#;(define (f32vector-mtimes v1 width-1 height-1 v2 width-2 to)
   ;; In math, we'd write:
-  ;; 	A[n,m] and B[m,p]
-  ;;	n = the number of lines of A
+  ;; 	A[n, m] and B[m, p]
+  ;;	n = the number of lines of Ab
   ;;	m = the number of columns of A
   ;;	    the number of lines of B
   ;;	p = the number of columns of B
@@ -570,11 +655,11 @@
          (n height-1)
          (m width-1)
          (p width-2)
-         (n-cell (* n p))
-	 (ab (make-f32vector n-cell)))
+         #;(n-cell (* n p))
+	 #;(ab (make-f32vector n-cell)))
     (do ((i 0
 	    (+ i 1)))
-	((= i n) ab)
+	((= i n) to)
       (do ((j 0
               (+ j 1)))
           ((= j p))
@@ -582,12 +667,12 @@
              (k 0
                 (+ k 1)))
             ((= k m)
-             (f32vector-set! ab (+ (* i p) j) sub))
+             (f32vector-set! to (+ (* i p) j) sub))
           (set! sub
                 (+ sub (* (f32vector-ref a (+ (* i m) k))
                           (f32vector-ref b (+ (* k p) j))))))))))
 
-(define (f32vector-invert v)
+#;(define (f32vector-invert v)
   (let* ((n-cell (f32vector-length v))
 	 (to (make-f32vector n-cell)))
     (do ((i 0
@@ -596,8 +681,9 @@
       (f32vector-set! to i
                       (expt (f32vector-ref v i) -1)))))
 
-(define (f32vector->s32vector v)
-  (let* ((n-cell (f32vector-length v))
+;; in libguile-cv
+#;(define* (f32vector->s32vector v  v #:key (n-cell #f))
+  (let* ((n-cell (or n-cell (f32vector-length v)))
 	 (to (make-s32vector n-cell)))
     (do ((i 0
 	    (+ i 1)))
